@@ -235,15 +235,14 @@ fn expand_conversion_impl(
             let ident_variants = variants.iter().map(|v| v.match_arm(&ident));
             let self_variants = variants.iter().map(|v| v.ctor(quote!(Self)));
 
-            let redaction = if let (EventKind::Message, EventKindVariation::Full) = (kind, var) {
-                quote! {
-                    #ident::RoomRedaction(event) => Self::RoomRedaction(
-                        #ruma_events::room::redaction::SyncRedactionEvent::from(event),
-                    ),
-                }
-            } else {
-                TokenStream::new()
-            };
+            let redaction =
+                (*kind == EventKind::Message && *var == EventKindVariation::Full).then(|| {
+                    quote! {
+                        #ident::RoomRedaction(event) => Self::RoomRedaction(
+                            #ruma_events::room::redaction::SyncRedactionEvent::from(event),
+                        ),
+                    }
+                });
 
             Some(quote! {
                 impl From<#ident> for #sync {
@@ -255,8 +254,8 @@ fn expand_conversion_impl(
                                 },
                             )*
                             #redaction
-                            #ident::Custom(event) => {
-                                Self::Custom(#ruma_events::#sync_struct::from(event))
+                            #ident::_Custom(event) => {
+                                Self::_Custom(#ruma_events::#sync_struct::from(event))
                             },
                         }
                     }
@@ -274,15 +273,14 @@ fn expand_conversion_impl(
             let self_variants = variants.iter().map(|v| v.match_arm(quote!(Self)));
             let full_variants = variants.iter().map(|v| v.ctor(&full));
 
-            let redaction = if let (EventKind::Message, EventKindVariation::Sync) = (kind, var) {
-                quote! {
-                    Self::RoomRedaction(event) => {
-                        #full::RoomRedaction(event.into_full_event(room_id))
-                    },
-                }
-            } else {
-                TokenStream::new()
-            };
+            let redaction =
+                (*kind == EventKind::Message && *var == EventKindVariation::Sync).then(|| {
+                    quote! {
+                        Self::RoomRedaction(event) => {
+                            #full::RoomRedaction(event.into_full_event(room_id))
+                        },
+                    }
+                });
 
             Some(quote! {
                 #[automatically_derived]
@@ -299,8 +297,8 @@ fn expand_conversion_impl(
                                 },
                             )*
                             #redaction
-                            Self::Custom(event) => {
-                                #full::Custom(event.into_full_event(room_id))
+                            Self::_Custom(event) => {
+                                #full::_Custom(event.into_full_event(room_id))
                             },
                         }
                     }
@@ -388,7 +386,8 @@ fn expand_content_enum(
                 #variant_decls(#content),
             )*
             /// Content of an event not defined by the Matrix specification.
-            Custom(#ruma_events::custom::CustomEventContent),
+            #[doc(hidden)]
+            _Custom(#ruma_events::custom::CustomEventContent),
         }
     };
 
@@ -405,7 +404,7 @@ fn expand_content_enum(
             fn event_type(&self) -> &::std::primitive::str {
                 match self {
                     #( #variant_arms(content) => content.event_type(), )*
-                    Self::Custom(content) => content.event_type(),
+                    Self::_Custom(content) => content.event_type(),
                 }
             }
 
@@ -423,7 +422,7 @@ fn expand_content_enum(
                     ev_type => {
                         let content =
                             #ruma_events::custom::CustomEventContent::from_parts(ev_type, input)?;
-                        ::std::result::Result::Ok(Self::Custom(content))
+                        ::std::result::Result::Ok(Self::_Custom(content))
                     },
                 }
             }
@@ -451,7 +450,8 @@ fn expand_content_enum(
                     #variant_decls(#redacted_content),
                 )*
                 /// Content of a redacted event not defined by the Matrix specification.
-                Custom(#ruma_events::custom::RedactedCustomEventContent),
+                #[doc(hidden)]
+                _Custom(#ruma_events::custom::RedactedCustomEventContent),
             }
 
             impl #ruma_events::RedactContent for #ident {
@@ -470,8 +470,8 @@ fn expand_content_enum(
                                 )
                             },
                         )*
-                        Self::Custom(content) => {
-                            #redacted_ident::Custom(
+                        Self::_Custom(content) => {
+                            #redacted_ident::_Custom(
                                 #ruma_events::RedactContent::redact(content, version)
                             )
                         },
@@ -559,9 +559,9 @@ fn expand_redact(
                                 })
                             }
                         )*
-                        Self::Custom(event) => {
+                        Self::_Custom(event) => {
                             let content = #ruma_events::RedactContent::redact(event.content, version);
-                            #redacted_enum::Custom(#redacted_type {
+                            #redacted_enum::_Custom(#redacted_type {
                                 content,
                                 #fields
                             })
@@ -585,14 +585,7 @@ fn expand_redacted_enum(
 
     if let EventKind::State | EventKind::Message = kind {
         let ident = format_ident!("AnyPossiblyRedacted{}", kind.to_event_ident(var)?);
-
-        // FIXME: Use Option::zip once MSRV >= 1.46
-        let (regular_enum_ident, redacted_enum_ident) = match inner_enum_idents(kind, var) {
-            (Some(regular_enum_ident), Some(redacted_enum_ident)) => {
-                (regular_enum_ident, redacted_enum_ident)
-            }
-            _ => return None,
-        };
+        let (regular_enum_ident, redacted_enum_ident) = inner_enum_idents(kind, var)?;
 
         Some(quote! {
             /// An enum that holds either regular un-redacted events or redacted events.
@@ -630,7 +623,7 @@ fn expand_redacted_enum(
 }
 
 fn generate_event_idents(kind: &EventKind, var: &EventKindVariation) -> Option<(Ident, Ident)> {
-    Some((kind.to_event_ident(var)?, kind.to_event_enum_ident(var)?))
+    kind.to_event_ident(var).zip(kind.to_event_enum_ident(var))
 }
 
 fn generate_redacted_fields(
@@ -639,24 +632,20 @@ fn generate_redacted_fields(
     var: &EventKindVariation,
     is_event_kind: EventKindFn,
     ruma_events: &TokenStream,
-) -> TokenStream {
-    if is_event_kind(kind, var) {
+) -> Option<TokenStream> {
+    is_event_kind(kind, var).then(|| {
         let name = Ident::new(name, Span::call_site());
 
         if name == "unsigned" {
             quote! {
-                unsigned: #ruma_events::RedactedUnsigned {
-                    redacted_because: Some(::std::boxed::Box::new(redaction)),
-                },
+                unsigned: #ruma_events::RedactedUnsigned::new_because(::std::boxed::Box::new(redaction)),
             }
         } else {
             quote! {
                 #name: event.#name,
             }
         }
-    } else {
-        TokenStream::new()
-    }
+    })
 }
 
 fn generate_custom_variant(
@@ -672,7 +661,8 @@ fn generate_custom_variant(
         (
             quote! {
                 /// A redacted event not defined by the Matrix specification
-                Custom(
+                #[doc(hidden)]
+                _Custom(
                     #ruma_events::#event_struct<#ruma_events::custom::RedactedCustomEventContent>,
                 ),
             },
@@ -683,7 +673,7 @@ fn generate_custom_variant(
                     >>(json.get())
                     .map_err(D::Error::custom)?;
 
-                    Ok(Self::Custom(event))
+                    Ok(Self::_Custom(event))
                 },
             },
         )
@@ -691,7 +681,8 @@ fn generate_custom_variant(
         (
             quote! {
                 /// An event not defined by the Matrix specification
-                Custom(#ruma_events::#event_struct<#ruma_events::custom::CustomEventContent>),
+                #[doc(hidden)]
+                _Custom(#ruma_events::#event_struct<#ruma_events::custom::CustomEventContent>),
             },
             quote! {
                 event => {
@@ -701,7 +692,7 @@ fn generate_custom_variant(
                         >(json.get())
                         .map_err(D::Error::custom)?;
 
-                    Ok(Self::Custom(event))
+                    Ok(Self::_Custom(event))
                 },
             },
         )
@@ -756,7 +747,7 @@ fn accessor_methods(
             match self {
                 #( #self_variants(event) =>
                     #ruma_events::EventContent::event_type(&event.content), )*
-                Self::Custom(event) =>
+                Self::_Custom(event) =>
                     #ruma_events::EventContent::event_type(&event.content),
             }
         }
@@ -767,12 +758,12 @@ fn accessor_methods(
         pub fn content(&self) -> #content_enum {
             match self {
                 #( #self_variants(event) => #content_variants(event.content.clone()), )*
-                Self::Custom(event) => #content_enum::Custom(event.content.clone()),
+                Self::_Custom(event) => #content_enum::_Custom(event.content.clone()),
             }
         }
     };
 
-    let prev_content = if has_prev_content_field(kind, var) {
+    let prev_content = has_prev_content_field(kind, var).then(|| {
         quote! {
             /// Returns the any content enum for this events prev_content.
             pub fn prev_content(&self) -> Option<#content_enum> {
@@ -782,15 +773,13 @@ fn accessor_methods(
                             event.prev_content.as_ref().map(|c| #content_variants(c.clone()))
                         },
                     )*
-                    Self::Custom(event) => {
-                        event.prev_content.as_ref().map(|c| #content_enum::Custom(c.clone()))
+                    Self::_Custom(event) => {
+                        event.prev_content.as_ref().map(|c| #content_enum::_Custom(c.clone()))
                     },
                 }
             }
         }
-    } else {
-        TokenStream::new()
-    };
+    });
 
     Some(quote! {
         #[automatically_derived]
@@ -806,22 +795,22 @@ fn accessor_methods(
     })
 }
 
-fn inner_enum_idents(kind: &EventKind, var: &EventKindVariation) -> (Option<Ident>, Option<Ident>) {
-    match var {
-        EventKindVariation::Full => {
-            (kind.to_event_enum_ident(var), kind.to_event_enum_ident(&EventKindVariation::Redacted))
-        }
+fn inner_enum_idents(kind: &EventKind, var: &EventKindVariation) -> Option<(Ident, Ident)> {
+    Some(match var {
+        EventKindVariation::Full => (
+            kind.to_event_enum_ident(var)?,
+            kind.to_event_enum_ident(&EventKindVariation::Redacted)?,
+        ),
         EventKindVariation::Sync => (
-            kind.to_event_enum_ident(var),
-            kind.to_event_enum_ident(&EventKindVariation::RedactedSync),
+            kind.to_event_enum_ident(var)?,
+            kind.to_event_enum_ident(&EventKindVariation::RedactedSync)?,
         ),
         EventKindVariation::Stripped => (
-            kind.to_event_enum_ident(var),
-            kind.to_event_enum_ident(&EventKindVariation::RedactedStripped),
+            kind.to_event_enum_ident(var)?,
+            kind.to_event_enum_ident(&EventKindVariation::RedactedStripped)?,
         ),
-        EventKindVariation::Initial => (kind.to_event_enum_ident(var), None),
-        _ => (None, None),
-    }
+        _ => return None,
+    })
 }
 
 /// Redacted events do NOT generate `content` or `prev_content` methods like
@@ -956,8 +945,8 @@ fn generate_accessor(
     is_event_kind: EventKindFn,
     variants: &[EventEnumVariant],
     ruma_events: &TokenStream,
-) -> TokenStream {
-    if is_event_kind(kind, var) {
+) -> Option<TokenStream> {
+    is_event_kind(kind, var).then(|| {
         let docs = format!("Returns this event's {} field.", name);
         let ident = Ident::new(name, Span::call_site());
         let field_type = field_return_type(name, var, ruma_events);
@@ -968,13 +957,11 @@ fn generate_accessor(
             pub fn #ident(&self) -> &#field_type {
                 match self {
                     #( #variants(event) => &event.#ident, )*
-                    Self::Custom(event) => &event.#ident,
+                    Self::_Custom(event) => &event.#ident,
                 }
             }
         }
-    } else {
-        TokenStream::new()
-    }
+    })
 }
 
 fn field_return_type(
